"use strict";
var Promise = require("bluebird");
var common_1 = require("./common");
var childLogger_1 = require("./childLogger");
var RabbitMqConsumer = (function () {
    function RabbitMqConsumer(logger, connectionFactory) {
        this.logger = logger;
        this.connectionFactory = connectionFactory;
        this.logger = childLogger_1.createChildLogger(logger, "RabbitMqConsumer");
    }
    RabbitMqConsumer.prototype.subscribe = function (queue, action) {
        var _this = this;
        var queueConfig = common_1.asQueueNameConfig(queue);
        return this.connectionFactory.create()
            .then(function (connection) { return connection.createChannel(); })
            .then(function (channel) {
            _this.logger.trace("got channel for queue '%s'", queueConfig.name);
            return _this.setupChannel(channel, queueConfig)
                .then(function () { return _this.subscribeToChannel(channel, queueConfig, action); });
        });
    };
    RabbitMqConsumer.prototype.setupChannel = function (channel, queueConfig) {
        this.logger.trace("setup '%j'", queueConfig);
        return Promise.all(this.getChannelSetup(channel, queueConfig));
    };
    RabbitMqConsumer.prototype.subscribeToChannel = function (channel, queueConfig, action) {
        var _this = this;
        this.logger.trace("subscribing to queue '%s'", queueConfig.name);
        return channel.consume(queueConfig.name, function (message) {
            var msg;
            Promise.try(function () {
                msg = _this.getMessageObject(message);
                _this.logger.trace("message arrived from queue '%s' (%j)", queueConfig.name, msg);
                return action(msg);
            }).then(function () {
                _this.logger.trace("message processed from queue '%s' (%j)", queueConfig.name, msg);
                channel.ack(message);
            }).catch(function (err) {
                _this.logger.error(err, "message processing failed from queue '%j' (%j)", queueConfig, msg);
                channel.nack(message, false, false);
            });
        }).then(function (opts) {
            _this.logger.trace("subscribed to queue '%s' (%s)", queueConfig.name, opts.consumerTag);
            return (function () {
                _this.logger.trace("disposing subscriber to queue '%s' (%s)", queueConfig.name, opts.consumerTag);
                return Promise.resolve(channel.cancel(opts.consumerTag)).return();
            });
        });
    };
    RabbitMqConsumer.prototype.getMessageObject = function (message) {
        return JSON.parse(message.content.toString('utf8'));
    };
    RabbitMqConsumer.prototype.getChannelSetup = function (channel, queueConfig) {
        return [
            channel.assertQueue(queueConfig.name, this.getQueueSettings(queueConfig.dlx)),
            channel.assertQueue(queueConfig.dlq, this.getDLSettings()),
            channel.assertExchange(queueConfig.dlx, 'fanout', this.getDLSettings()),
            channel.bindQueue(queueConfig.dlq, queueConfig.dlx, '*')
        ];
    };
    RabbitMqConsumer.prototype.getQueueSettings = function (deadletterExchangeName) {
        var settings = this.getDLSettings();
        settings.arguments = {
            'x-dead-letter-exchange': deadletterExchangeName
        };
        return settings;
    };
    RabbitMqConsumer.prototype.getDLSettings = function () {
        return {
            durable: true,
            autoDelete: false
        };
    };
    return RabbitMqConsumer;
}());
exports.RabbitMqConsumer = RabbitMqConsumer;
//# sourceMappingURL=consumer.js.map